`timescale 1ns / 1ps
//////////////////////////////////////////////////////////////////////////////////
// Company: 
// Engineer: 
// 
// Create Date:    23:26:53 10/13/2011 
// Design Name: 
// Module Name:    text_store_controller 
// Project Name: 
// Target Devices: 
// Tool versions: 
// Description: 
//
// Dependencies: 
//
// Revision: 
// Revision 0.01 - File Created
// Additional Comments: 
//
//////////////////////////////////////////////////////////////////////////////////
module text_store_controller(
    input wr_enb,
    input dat_clk,
    input sys_clk,
	 input recycle,
	 output reg end_write,
    output wr_ack,
    output wr_rdy,
    output wen,
	 output reg [7:0] mem_addr
    );
reg dclk_synchd;
reg [3:0] pstate_control, nstate_control;
parameter [3:0] rest = 4'b0000,	// Define states of address controller
	rdy = 4'b0001, 
	start = 4'b0011,
	clock_tick = 4'b1011, 
	wait_done = 4'b0110, 
	next_char = 4'b0010;
parameter [7:0] max_count = 8'b1001100;
	
always @ (posedge sys_clk) begin		// All registers and synchronizer
	dclk_synchd <= dat_clk;
	pstate_control <= nstate_control;
	if (wr_enb == 1'b0) mem_addr <= rest;
	else if (mem_addr == max_count) mem_addr <= max_count;
	else if (wen == 1'b1) mem_addr <= mem_addr + 1;
	else mem_addr <= mem_addr;
	end
	
always @ (*)  begin	// Next state logic
	case (pstate_control)
	rest: if (wr_enb == 1'b0) nstate_control = rest; else nstate_control = rdy;
	rdy: if (wr_enb == 1'b0) nstate_control = rest; 
			else if (dclk_synchd == 0) nstate_control = start; 
			else nstate_control = rdy;
	start: if (wr_enb == 1'b0) nstate_control = rest; 
			else if (dclk_synchd == 1'b1) nstate_control = clock_tick;
			else nstate_control = start;
	clock_tick: if (wr_enb == 1'b0) nstate_control = rest;
			else if (mem_addr == max_count) nstate_control = wait_done;
			else nstate_control = next_char;
	next_char: if (wr_enb == 1'b0) nstate_control = rest;
			else if (recycle == 1'b1) nstate_control = wait_done;
			else if (dclk_synchd == 1'b0) nstate_control = start;
			else nstate_control = next_char;
	wait_done: if (wr_enb == 1'b0) nstate_control = rest;
			else nstate_control = wait_done;
	default: nstate_control = rest;
	endcase
	end
	
assign	wen = pstate_control[3];
assign	wr_ack = ~pstate_control[0] & pstate_control[1] & ~pstate_control[2];
assign	wr_rdy = pstate_control[0] | pstate_control[1] | pstate_control[2] | pstate_control[3];
always @ (pstate_control) if (pstate_control == wait_done) end_write = 1'b1; else end_write = 1'b0;

endmodule
